package HomeWork.system.api;

import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;
import HomeWork.common.core.constant.SecurityConstants;
import HomeWork.common.core.constant.ServiceNameConstants;
import HomeWork.common.core.domain.R;
import HomeWork.system.api.domain.SysUser;
import HomeWork.system.api.factory.RemoteUserFallbackFactory;
import HomeWork.system.api.model.LoginUser;

/**
 * 用户服务
 *
 * @author ruoyi
 */
@FeignClient(contextId = "remoteUserService", value = ServiceNameConstants.SYSTEM_SERVICE, fallbackFactory = RemoteUserFallbackFactory.class)
public interface RemoteUserService
{
    /**
     * 通过用户名查询用户信息
     *
     * @param username 用户名
     * @param source 请求来源
     * @return 结果
     */
    @GetMapping("/user/info/{username}")
    public R<LoginUser> getUserInfo(@PathVariable("username") String username, @RequestHeader(SecurityConstants.FROM_SOURCE) String source);

    /**
     * 注册用户信息
     *
     * @param sysUser 用户信息
     * @param source 请求来源
     * @return 结果
     */
    @PostMapping("/user/register")
    public R<Boolean> registerUserInfo(@RequestBody SysUser sysUser, @RequestHeader(SecurityConstants.FROM_SOURCE) String source);

    /**
     * 人脸登录
     * @param url
     * @return
     * @throws Exception
     */
    @PostMapping("/user/faceLogin")
    public R<LoginUser> faceLogin(@RequestParam("url") String url, @RequestHeader(SecurityConstants.FROM_SOURCE) String source);

}
