package HomeWork.auth.controller;

import javax.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import HomeWork.auth.form.LoginBody;
import HomeWork.auth.form.RegisterBody;
import HomeWork.auth.service.SysLoginService;
import HomeWork.common.core.domain.R;
import HomeWork.common.core.utils.JwtUtils;
import HomeWork.common.core.utils.StringUtils;
import HomeWork.common.security.auth.AuthUtil;
import HomeWork.common.security.service.TokenService;
import HomeWork.common.security.utils.SecurityUtils;
import HomeWork.system.api.model.LoginUser;
import org.springframework.web.multipart.MultipartFile;

/**
 * token 控制
 *
 * @author ruoyi
 */
@RestController
public class TokenController
{
    @Autowired
    private TokenService tokenService;

    @Autowired
    private SysLoginService sysLoginService;

    @PostMapping("login")
    public R<?> login(@RequestBody LoginBody form)
    {
        // 用户登录
        LoginUser userInfo = sysLoginService.login(form.getUsername(), form.getPassword());
        // 获取登录token
        return R.ok(tokenService.createToken(userInfo));
    }

    @PostMapping("/faceLogin")
    public R<?> login(@RequestParam MultipartFile file) throws Exception {
        // 用户登录
        LoginUser userInfo = sysLoginService.faceLogin(file);
        if (userInfo==null){
            return R.fail("人脸登录失败");
        }
        // 获取登录token
        return R.ok(tokenService.createToken(userInfo));
    }



    @DeleteMapping("logout")
    public R<?> logout(HttpServletRequest request)
    {
        String token = SecurityUtils.getToken(request);
        if (StringUtils.isNotEmpty(token))
        {
            String username = JwtUtils.getUserName(token);
            // 删除用户缓存记录
            AuthUtil.logoutByToken(token);
            // 记录用户退出日志
            sysLoginService.logout(username);
        }
        return R.ok();
    }

    @PostMapping("refresh")
    public R<?> refresh(HttpServletRequest request)
    {
        LoginUser loginUser = tokenService.getLoginUser(request);
        if (StringUtils.isNotNull(loginUser))
        {
            // 刷新令牌有效期
            tokenService.refreshToken(loginUser);
            return R.ok();
        }
        return R.ok();
    }

    @PostMapping("register")
    public R<?> register(@RequestBody RegisterBody registerBody)
    {
        // 用户注册
        sysLoginService.register(registerBody.getUsername(), registerBody.getPassword());
        return R.ok();
    }
}
