package HomeWork.service.impl;

import HomeWork.common.core.domain.R;
import HomeWork.domain.FloorEntity;
import HomeWork.domain.TreeEntity;
import HomeWork.domain.UnitEntity;
import HomeWork.domain.dto.PlotDto;
import HomeWork.domain.vo.PlotVo;
import HomeWork.domain.vo.TreeVo;
import HomeWork.domain.vo.UnitVo;
import HomeWork.mapper.TreeMapper;
import HomeWork.service.TreeService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Service
public class TreeServiceImpl implements TreeService {
    @Autowired
    private TreeMapper treeMapper;

    private final static String PARENT_NAME="根节点";
    @Override
    public List<PlotVo> plotList(Integer id) {
        return treeMapper.plotList(id);
    }

    @Override
    public R add(PlotDto plotDto) {
        if (treeMapper.add(plotDto)==0){
            return R.fail("添加失败");
        }
        return R.ok("添加成功");
    }

    @Override
    public List<TreeVo> treeListByTreeParentId(Integer parentId) {
        if (parentId!=0){
            //返回体
            ArrayList<TreeVo> treeVos = new ArrayList<>();
            //通过父级id查询所有的对象用所有的对象的id查询所有的子节点
            List<TreeEntity> treeEntities = treeMapper.treeByParentId(parentId);
            //将所有的id拼接成字符串
            String ids="";
            //循环拼接
            for (TreeEntity treeEntity : treeEntities) {
                TreeVo treeVo = new TreeVo();
                BeanUtils.copyProperties(treeEntity,treeVo);
                treeVos.add(treeVo);
                ids+=","+treeEntity.getId();
            }
            //切割字符循环查找
            String[] split = Arrays.stream(ids.split(","))
                    .filter(s -> !s.isEmpty())
                    .toArray(String[]::new);

            //将集合转型返回
            List<TreeEntity> treeVos1 = treeMapper.treeListByTreeParentIds(split);
            for (TreeEntity tree : treeVos1) {

                TreeVo treeVo = new TreeVo();
                BeanUtils.copyProperties(tree,treeVo);
                treeVos.add(treeVo);
            }
            return treeVos;
        }else {
            //返回体
            List<TreeVo> treeVos = treeList();
            return treeVos;
        }
    }

    @Override
    public R file(MultipartFile file) {
        return null;
    }

    @Override
    public List<UnitVo> unitListById(String id) {
        return null;
    }

    @Override
    public List<TreeVo> treeList() {
        ArrayList<TreeVo> treeVos = new ArrayList<>();
        List<TreeEntity> treeVoList = treeMapper.treeList();
        if (treeVoList.size()>0){
            for (TreeEntity tree : treeVoList) {
                TreeVo treeVo = new TreeVo();
                BeanUtils.copyProperties(tree,treeVo);
                List<TreeVo> children = findChildren(tree.getId());
                treeVo.setChildren(children);
                treeVos.add(treeVo);
            }
        }
        return treeVos;
    }

    private List<TreeVo> findChildren(Integer id) {
        ArrayList<TreeVo> treeVos = new ArrayList<>();
        List<TreeEntity> treeEntities = treeMapper.findChildren(id);
        String ids="";
        for (TreeEntity treeVo : treeEntities) {
            ids+=treeVo.getId()+",";
        }
        String[] split = ids.split(",");
        List<TreeEntity> treeVos1 = treeMapper.treeListByTreeIds(split);
        for (TreeEntity treeVo : treeVos1) {
            TreeVo treeVo1 = new TreeVo();
            BeanUtils.copyProperties(treeVo,treeVo1);
            List<TreeEntity> children = treeMapper.findChildren(treeVo1.getId());
            if (children.size()>0){
                treeVo1.setChildren(findChildren(treeVo1.getId()));
            }
            treeVos.add(treeVo1);
        }
        return treeVos;
    }/**/

}
