package com.bawei.es.server.impl;

import com.alibaba.fastjson.JSONObject;
import com.bawei.es.domain.DocumentReq;
import com.bawei.es.domain.IndexEntry;
import com.bawei.es.domain.IndexInfoReq;
import com.bawei.es.domain.IndexInfoResp;
import com.bawei.es.server.EsIndexService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.action.admin.indices.delete.DeleteIndexRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.indices.GetIndexRequest;
import org.elasticsearch.client.indices.GetIndexResponse;
import org.elasticsearch.client.indices.PutMappingRequest;
import org.elasticsearch.cluster.metadata.AliasMetadata;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.xcontent.XContentType;
import org.springframework.stereotype.Service;
import org.elasticsearch.action.admin.indices.create.CreateIndexRequest;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Service
@Slf4j
@AllArgsConstructor
public class EsIndexServiceImpl implements EsIndexService {

    private RestHighLevelClient restHighLevelClient;

    /**
     * 创建索引
     * @param indexInfoReq
     */
    @Override
    public void createIndex(IndexInfoReq indexInfoReq) {
        //打印日志
        log.info("创建索引{}", JSONObject.toJSONString(indexInfoReq));
        //判断名字是否存在
        boolean existIndex = isExistIndex(indexInfoReq.getIndexName());
        if (existIndex){
            throw new RuntimeException("索引:"+indexInfoReq.getIndexName()+"索引已存在");
        }
        //创建索引
        CreateIndexRequest createIndexRequest = new CreateIndexRequest(indexInfoReq.getIndexName());
        if(indexInfoReq.getIndexMapping()!=null){
            String jsonString = JSONObject.toJSONString(indexInfoReq.getIndexMapping());
            createIndexRequest.mapping(indexInfoReq.getIndexName(),jsonString,XContentType.JSON);
        }
        try {
            restHighLevelClient.indices().create(createIndexRequest, RequestOptions.DEFAULT);
        } catch (Exception e) {
            throw new RuntimeException("创建失败",e);
        }
    }
    @Override
    public void delIndex(String indexName) {
        try {
            boolean existIndex = isExistIndex(indexName);
            if (!existIndex){
                throw new RuntimeException("索引:"+indexName+"不存在");
            }
            DeleteIndexRequest deleteIndexRequest = new DeleteIndexRequest(indexName);
            restHighLevelClient.indices().delete(deleteIndexRequest,RequestOptions.DEFAULT);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 判断索引是否存在
     * @param indexName
     * @return
     */
    public boolean isExistIndex(String indexName) {
        GetIndexRequest getIndexResult = new GetIndexRequest(indexName);
        try {
            boolean exists = restHighLevelClient.indices().exists(getIndexResult, RequestOptions.DEFAULT);
            return exists;
        } catch (Exception e) {
            throw new RuntimeException("查询索引失败",e);
        }
    }

    /**
     * 查找索引
     * @param indexName
     * @return
     */
    @Override
    public IndexInfoResp find(String indexName) {
        if(isExistIndex(indexName)){
            IndexInfoResp indexInfoResp = new IndexInfoResp();
            GetIndexRequest getIndexRequest = new GetIndexRequest(indexName);
            try {
                IndexInfoResp indexInfoResp1 = new IndexInfoResp();
                GetIndexResponse getIndexResponse = restHighLevelClient.indices().get(getIndexRequest, RequestOptions.DEFAULT);
                indexInfoResp1.setIndexName(getIndexResponse.getIndices()[0]);
                indexInfoResp1.setMappings(getIndexResponse.getMappings().get(indexName).getSourceAsMap());
                Settings settings = getIndexResponse.getSettings().get(indexName);
                Set<String> strings = settings.keySet();
                HashMap<String, Object> map = new HashMap<>();
                strings.forEach(key->{
                    map.put(key,settings.get(key));
                });
                indexInfoResp1.setSettings(map);
                Map<String, List<AliasMetadata>> aliases = getIndexResponse.getAliases();
                Map<String,Object> aliasMap=JSONObject.parseObject(JSONObject.toJSONString(aliases),Map.class);
                indexInfoResp.setAliases(aliasMap);
                return indexInfoResp;
            } catch (Exception e) {
                throw new RuntimeException("索引失败",e);
            }
        }else {
            throw new RuntimeException("索引名不存在");
        }
    }

    @Override
    public void updIndex(IndexEntry indexEntry) {
        try {
            if (isExistIndex(indexEntry.getIndexName())){
                PutMappingRequest putMappingRequest = new PutMappingRequest(indexEntry.getIndexName());
                String jsonString = JSONObject.toJSONString(indexEntry.getIndexMapping());
                putMappingRequest.source(jsonString,XContentType.JSON);
                restHighLevelClient.indices().putMapping(putMappingRequest,RequestOptions.DEFAULT);
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
